/* Teensyduino Core Library
 * http://www.pjrc.com/teensy/
 * Copyright (c) 2013 PJRC.COM, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * 1. The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 *
 * 2. If the Software is incorporated into a build system that allows 
 * selection among a list of target devices, then similar target
 * devices manufactured by PJRC.COM must be included in the list of
 * target devices and selectable in the same manner.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef USBjoystick_h_
#define USBjoystick_h_

#if defined(USB_HID) || defined(USB_SERIAL_HID)

#include <inttypes.h>

// C language implementation
#ifdef __cplusplus
extern "C" {
#endif
int usb_joystick_send(void);
extern uint32_t usb_joystick_data[6];
#ifdef __cplusplus
}
#endif

// C++ interface
#ifdef __cplusplus
class usb_joystick_class
{
        public:
        void begin(void) { }
        void end(void) { }
	void button(uint8_t button, bool val) {
		if (--button >= 32) return;
		if (val) usb_joystick_data[0] |= (1 << button);
		else usb_joystick_data[0] &= ~(1 << button);
		if (!manual_mode) usb_joystick_send();
	}
	inline void hat(int dir) {
		uint32_t val;
		if (dir < 0) val = 15;
		else if (dir < 23) val = 0;
		else if (dir < 68) val = 1;
		else if (dir < 113) val = 2;
		else if (dir < 158) val = 3;
		else if (dir < 203) val = 4;
		else if (dir < 245) val = 5;
		else if (dir < 293) val = 6;
		else if (dir < 338) val = 7;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0xFFFFFFF0) | val;
		if (!manual_mode) usb_joystick_send();
	}
	inline void hat2(int dir) {
		uint32_t val;
		if (dir < 0) val = 15;
		else if (dir < 23) val = 0;
		else if (dir < 68) val = 1;
		else if (dir < 113) val = 2;
		else if (dir < 158) val = 3;
		else if (dir < 203) val = 4;
		else if (dir < 245) val = 5;
		else if (dir < 293) val = 6;
		else if (dir < 338) val = 7;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0xFFFFFF0F) | (val << 4);
		if (!manual_mode) usb_joystick_send();
	}
	inline void hat3(int dir) {
		uint32_t val;
		if (dir < 0) val = 15;
		else if (dir < 23) val = 0;
		else if (dir < 68) val = 1;
		else if (dir < 113) val = 2;
		else if (dir < 158) val = 3;
		else if (dir < 203) val = 4;
		else if (dir < 245) val = 5;
		else if (dir < 293) val = 6;
		else if (dir < 338) val = 7;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0xFFFFF0FF) | (val << 8);
		if (!manual_mode) usb_joystick_send();
	}
	inline void hat4(int dir) {
		uint32_t val;
		if (dir < 0) val = 15;
		else if (dir < 23) val = 0;
		else if (dir < 68) val = 1;
		else if (dir < 113) val = 2;
		else if (dir < 158) val = 3;
		else if (dir < 203) val = 4;
		else if (dir < 245) val = 5;
		else if (dir < 293) val = 6;
		else if (dir < 338) val = 7;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0xFFFF0FFF) | (val << 12);
		if (!manual_mode) usb_joystick_send();
	}
	void X(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0xFFFF) | (val << 16);
		if (!manual_mode) usb_joystick_send();
	}
	void Y(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[2] = (usb_joystick_data[2] & 0xffff0000) | val;
		if (!manual_mode) usb_joystick_send();
	}
	void position(uint32_t x, uint32_t y) {
		if (x > 65535) x = 65535;
		if (y > 65535) y = 65535;
		usb_joystick_data[1] = (usb_joystick_data[1] & 0xFFFF) | (x << 16);
		usb_joystick_data[2] = (usb_joystick_data[2] & 0xffff0000) | y;	
		if (!manual_mode) usb_joystick_send();
	}
	void Z(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[2] = (usb_joystick_data[2] & 0xFFFF) | (val << 16);
		if (!manual_mode) usb_joystick_send();
	}
	void Xrotate(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[3] = (usb_joystick_data[3] & 0xFFFF0000) | val;
		if (!manual_mode) usb_joystick_send();
	}
	void Yrotate(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[3] = (usb_joystick_data[3] & 0xFFFF) | (val << 16);
		if (!manual_mode) usb_joystick_send();
	}
	void Zrotate(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[4] = (usb_joystick_data[4] & 0xFFFF0000) | val;
		if (!manual_mode) usb_joystick_send();
	}
	
	void sliderRight(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[4] = (usb_joystick_data[4] & 0xFFFF) | (val << 16);
		if (!manual_mode) usb_joystick_send();
	}
	
	void sliderLeft(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[5] = (usb_joystick_data[5] & 0xFFFF0000) | val;
		if (!manual_mode) usb_joystick_send();
	}
	
	void slider(uint32_t val) {
		if (val > 65535) val = 65535;
		usb_joystick_data[4] = (usb_joystick_data[4] & 0xFFFF) | (val << 16);
		usb_joystick_data[5] = (usb_joystick_data[5] & 0xFFFF0000) | val;
		if (!manual_mode) usb_joystick_send();
	}
	
	void useManualSend(bool mode) {
		manual_mode = mode;
	}
	void send_now(void) {
		usb_joystick_send();
	}
	private:
	static uint8_t manual_mode;
};
extern usb_joystick_class Joystick;

#endif // __cplusplus

#endif // USB_HID || USB_SERIAL_HID
#endif // USBjoystick_h_

